@@ -1,6 +1,4 @@
 #!/usr/bin/env python3
-# -*- coding: utf-8 -*-
-# coding=UTF-8
 # Relational
 # Copyright (C) 2010  Salvo "LtWorf" Tomaselli
 #
@@ -21,6 +19,8 @@
 
 import os
 from sys import exit
+import sys
+import traceback
 
 from relational import relation, parser, optimizer
 from xtermcolor import colorize
@@ -40,7 +40,7 @@ tests_path = 'test/'
 
 def readfile(fname):
     '''Reads a file as string and returns its content'''
-    fd = open(fname)
+    fd = open(fname, encoding='utf-8')
     expr = fd.read()
     fd.close()
     return expr
@@ -87,9 +87,9 @@ def execute_tests():
                 py_good += 1
             else:
                 py_bad += 1
-        elif i.endswith('.exec'):
+        elif i.endswith('.py'):
             ex_tot += 1
-            if run_exec_test(i[:-5]):
+            if run_exec_test(i[:-3]):
                 ex_good += 1
             else:
                 ex_bad += 1
@@ -130,34 +130,18 @@ def run_exec_test(testname):
     glob = rels.copy()
     exp_result = {}
 
+    expr = readfile('%s%s.py' % (tests_path, testname))
+
     try:
-        expr = readfile('%s%s.exec' % (tests_path, testname))
-        try:
-            exec(expr, glob)  # Evaluating the expression
-        except Exception as e:
-            print (e)
-            raise Exception("")
-
-        expr = readfile('%s%s.result' % (tests_path, testname))
-        exp_result = eval(expr, rels)  # Evaluating the expression
-
-        if isinstance(exp_result, dict):
-            fields_ok = True
-
-            for i in exp_result:
-                fields_ok = fields_ok and glob[i] == exp_result[i]
-
-            if fields_ok:
-                print (colorize('Test passed', COLOR_GREEN))
-                return True
-    except:
-        pass
-    print (colorize('ERROR', COLOR_RED))
-    print (colorize('=====================================', COLOR_RED))
-    print ("Expected %s" % exp_result)
-    # print ("Got %s" % glob)
-    print (colorize('=====================================', COLOR_RED))
-    return False
+        exec(expr, glob)  # Evaluating the expression
+        print (colorize('Test passed', COLOR_GREEN))
+        return True
+    except Exception as e:
+        print (colorize('ERROR', COLOR_RED))
+        print (colorize('=====================================', COLOR_RED))
+        traceback.print_exc(file=sys.stdout)
+        print (colorize('=====================================', COLOR_RED))
+        return False
 
 
 def run_py_test(testname):
@@ -168,10 +152,10 @@ def run_py_test(testname):
     try:
 
         expr = readfile('%s%s.python' % (tests_path, testname))
-        result = eval(expr, rels)  # Evaluating the expression
+        result = eval(expr, rels)
 
         expr = readfile('%s%s.result' % (tests_path, testname))
-        exp_result = eval(expr, rels)  # Evaluating the expression
+        exp_result = eval(expr, rels)
 
         if result == exp_result:
             print (colorize('Test passed', COLOR_GREEN))
@@ -210,20 +194,20 @@ def run_test(testname):
         query = readfile('%s%s.query' % (tests_path, testname)).strip()
         o_query = optimizer.optimize_all(query, rels)
 
-        expr = parser.parse(query)  # Converting expression to python string
-        result = eval(expr, rels)  # Evaluating the expression
+        expr = parser.parse(query)
+        result = expr(rels)
 
-        o_expr = parser.parse(
-            o_query)  # Converting expression to python string
-        o_result = eval(o_expr, rels)  # Evaluating the expression
+        o_expr = parser.parse(o_query)
+        o_result = o_expr(rels)
 
-        c_expr = parser.tree(query).toCode()  # Converting to python code
+        c_expr = parser.tree(query).toCode()
         c_result = eval(c_expr, rels)
 
         if (o_result == result_rel) and (result == result_rel) and (c_result == result_rel):
             print (colorize('Test passed', COLOR_GREEN))
             return True
     except Exception as inst:
+        traceback.print_exc(file=sys.stdout)
         print (inst)
         pass
     print (colorize('ERROR', COLOR_RED))
