@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Relational
 # Copyright (C) 2009  Salvo "LtWorf" Tomaselli
 #
@@ -157,7 +156,9 @@ def futile_union_intersection_subtraction(n):
             replace_node(n, n.right)
 
     # Subtraction and selection of the same thing
-    elif (n.name == DIFFERENCE and (n.right.name == SELECTION and n.right.child == n.left)):  # Subtraction of two equal things, but one has a selection
+    elif n.name == DIFFERENCE and \
+            n.right.name == SELECTION and \
+            n.right.child == n.left:
         n.name = n.right.name
         n.kind = n.right.kind
         n.child = n.right.child
@@ -165,7 +166,7 @@ def futile_union_intersection_subtraction(n):
         n.left = n.right = None
 
     # Subtraction of the same thing or with selection on the left child
-    elif (n.name == DIFFERENCE and ((n.left == n.right) or (n.left.name == SELECTION and n.left.child == n.right))):  # Empty relation
+    elif n.name == DIFFERENCE and (n.left == n.right or (n.left.name == SELECTION and n.left.child == n.right)):
         changes = 1
         n.kind = parser.UNARY
         n.name = SELECTION
@@ -289,22 +290,14 @@ def futile_renames(n):
         # Scans dictionary to locate things like "a->b,b->c" and replace them
         # with "a->c"
         for key in list(_vars.keys()):
-            try:
-                value = _vars[key]
-            except:
-                value = None
+            value = _vars.get(key)
             if key == value:
                 _vars.pop(value)  # Removes the unused one
-        # Reset prop var
-        n.prop = ""
 
-        # Generates new prop var
-        for i in _vars.items():
-            n.prop += u"%s%s%s," % (i[0], ARROW, i[1])
-        n.prop = n.prop[:-1]  # Removing ending comma
-
-        if len(n.prop) == 0:  # Nothing to rename, removing the rename op
+        if len(_vars) == 0: # Nothing to rename, removing the rename op
             replace_node(n, n.child)
+        else:
+            n.prop = ','.join('%s%s%s' % (i[0], ARROW, i[1]) for i in _vars.items())
 
     return changes + recoursive_scan(futile_renames, n)
 
@@ -317,7 +310,7 @@ def subsequent_renames(n):
     futile_renames(n)
     changes = 0
 
-    if n.name == RENAME and n.child.name == n.name:
+    if n.name == RENAME and n.child.name == RENAME:
         # Located two nested renames.
         changes = 1
         # Joining the attribute into one
@@ -332,10 +325,7 @@ def subsequent_renames(n):
         # Scans dictionary to locate things like "a->b,b->c" and replace them
         # with "a->c"
         for key in list(_vars.keys()):
-            try:
-                value = _vars[key]
-            except:
-                value = None
+            value = _vars.get(key)
             if value in _vars.keys():
                 if _vars[value] != key:
                     # Double rename on attribute
@@ -345,16 +335,10 @@ def subsequent_renames(n):
                     _vars.pop(value)  # Removes the unused one
                     _vars.pop(key)  # Removes the unused one
 
-        # Reset prop var
-        n.prop = ""
-
-        # Generates new prop var
-        for i in _vars.items():
-            n.prop += u"%s%s%s," % (i[0], ARROW, i[1])
-        n.prop = n.prop[:-1]  # Removing ending comma
-
-        if len(n.prop) == 0:  # Nothing to rename, removing the rename op
+        if len(_vars) == 0:  # Nothing to rename, removing the rename op
             replace_node(n, n.child)
+        else:
+            n.prop = ','.join('%s%s%s' % (i[0], ARROW, i[1]) for i in _vars.items())
 
     return changes + recoursive_scan(subsequent_renames, n)
 
@@ -426,10 +410,8 @@ def swap_rename_projection(n):
             if i not in _pr_reborn:
                 _vars.pop(i)
         n.name = n.child.name
-        n.prop = ''
-        for i in _vars.keys():
-            n.prop += u'%s%s%s,' % (_vars[i], ARROW, i)
-        n.prop = n.prop[:-1]
+
+        n.prop = ','.join('%s%s%s' % (i[1], ARROW, i[0]) for i in _vars.items())
 
         n.child.name = PROJECTION
         n.child.prop = ''
@@ -472,9 +454,7 @@ def swap_rename_select(n):
         n.child.name = SELECTION
 
         n.prop = n.child.prop
-        n.child.prop = ''
-        for i in _tokens:
-            n.child.prop += i + ' '
+        n.child.prop = ' '.join(_tokens)
 
     return changes + recoursive_scan(swap_rename_select, n)
 
@@ -484,8 +464,11 @@ def select_union_intersect_subtract(n):
     and replaces them with σ (i OR q) (a)
     Removing a O(n²) operation like the union'''
     changes = 0
-    if n.name in (UNION, INTERSECTION, DIFFERENCE) and n.left.name == SELECTION and n.right.name == SELECTION and n.left.child == n.right.child:
-        cahnges = 1
+    if n.name in {UNION, INTERSECTION, DIFFERENCE} and \
+                n.left.name == SELECTION and \
+                n.right.name == SELECTION and \
+                n.left.child == n.right.child:
+        changes = 1
 
         d = {UNION: 'or', INTERSECTION: 'and', DIFFERENCE: 'and not'}
         op = d[n.name]
@@ -516,6 +499,74 @@ def select_union_intersect_subtract(n):
     return changes + recoursive_scan(select_union_intersect_subtract, n)
 
 
+def union_and_product(n):
+    '''
+    A * B ∪ A * C = A * (B ∪ C)
+    Same thing with inner join
+    '''
+
+    changes = 0
+    if n.name == UNION and n.left.name in {PRODUCT, JOIN} and n.left.name == n.right.name:
+
+        newnode = parser.node()
+        newnode.kind = parser.BINARY
+        newnode.name = n.left.name
+
+        newchild = parser.node()
+        newchild.kind = parser.BINARY
+        newchild.name = UNION
+
+        if n.left.left == n.right.left or n.left.left == n.right.right:
+            newnode.left = n.left.left
+            newnode.right = newchild
+
+            newchild.left = n.left.right
+            newchild.right = n.right.left if n.left.left == n.right.right else n.right.right
+            replace_node(n, newnode)
+            changes = 1
+        elif n.left.right == n.right.left or n.left.left == n.right.right:
+            newnode.left = n.left.right
+            newnode.right = newchild
+
+            newchild.left = n.left.left
+            newchild.right = n.right.left if n.right.left == n.right.right else n.right.right
+            replace_node(n, newnode)
+            changes = 1
+    return changes + recoursive_scan(union_and_product, n)
+
+
+def projection_and_union(n, rels):
+    '''
+    Turns
+        π a,b,c(A) ∪ π a,b,c(B)
+
+    into
+        π a,b,c(A ∪ B)
+
+    if A and B are union compatible
+    '''
+    changes = 0
+    if n.name in {UNION, INTERSECTION, DIFFERENCE} and \
+            n.left.name == PROJECTION and \
+            n.right.name == PROJECTION and \
+            set(n.left.child.result_format(rels)) == set(n.right.child.result_format(rels)):
+        newchild = parser.Node()
+
+        newchild.kind = parser.BINARY
+        newchild.name = n.name
+        newchild.left = n.left.child
+        newchild.right = n.right.child
+
+        newnode = parser.Node()
+        newnode.child = newchild
+        newnode.kind = parser.UNARY
+        newnode.name = PROJECTION
+        newnode.prop = n.right.prop
+        replace_node(n, newnode)
+        changes = 1
+    return changes + recoursive_scan(projection_and_union, n, rels)
+
+
 def selection_and_product(n, rels):
     '''This function locates things like σ k (R*Q) and converts them into
     σ l (σ j (R) * σ i (Q)). Where j contains only attributes belonging to R,
@@ -593,8 +644,7 @@ def selection_and_product(n, rels):
             n.child.right = r_node
             while len(right) > 0:
                 c = right.pop(0)
-                for i in c:
-                    r_node.prop += i + ' '
+                r_node.prop += ' '.join(c)
                 if len(right) > 0:
                     r_node.prop += ' and '
             if '(' in r_node.prop:
@@ -604,8 +654,7 @@ def selection_and_product(n, rels):
         if len(both) != 0:
             while len(both) > 0:
                 c = both.pop(0)
-                for i in c:
-                    n.prop += i + ' '
+                n.prop += ' '.join(c)
                 if len(both) > 0:
                     n.prop += ' and '
             if '(' in n.prop:
@@ -615,10 +664,37 @@ def selection_and_product(n, rels):
 
     return changes + recoursive_scan(selection_and_product, n, rels)
 
+
+def useless_projection(n, rels):
+    '''
+    Removes projections that are over all the fields
+    '''
+    changes = 0
+    if n.name == PROJECTION and \
+            set(n.child.result_format(rels)) == set(i.strip() for i in n.prop.split(',')):
+        changes = 1
+        replace_node(n, n.child)
+
+    return changes + recoursive_scan(useless_projection, n, rels)
+
 general_optimizations = [
-    duplicated_select, down_to_unions_subtractions_intersections, duplicated_projection, selection_inside_projection,
-    subsequent_renames, swap_rename_select, futile_union_intersection_subtraction, swap_union_renames, swap_rename_projection, select_union_intersect_subtract]
-specific_optimizations = [selection_and_product]
+    duplicated_select,
+    down_to_unions_subtractions_intersections,
+    duplicated_projection,
+    selection_inside_projection,
+    subsequent_renames,
+    swap_rename_select,
+    futile_union_intersection_subtraction,
+    swap_union_renames,
+    swap_rename_projection,
+    select_union_intersect_subtract,
+    union_and_product,
+]
+specific_optimizations = [
+    selection_and_product,
+    projection_and_union,
+    useless_projection,
+]
 
 if __name__ == "__main__":
     print (tokenize_select("skill == 'C' and  id % 2 == 0"))
