@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-# coding=UTF-8
 # Relational
 # Copyright (C) 2008  Salvo "LtWorf" Tomaselli
 #
@@ -24,43 +22,27 @@
 # to convert expressions into python expressions and to get the parse-tree
 # of the expression.
 #
-# The input must be provided in UTF-8
-#
-#
-# Language definition:
-# Query := Ident
-# Query := Query BinaryOp Query
-# Query := (Query)
-# Query := σ PYExprWithoutParenthesis (Query) | σ (PYExpr) (Query)
-# Query := π FieldList (Query)
-# Query := ρ RenameList (Query)
-# FieldList := Ident | Ident , FieldList
-# RenameList := Ident ➡ Ident | Ident ➡ Ident , RenameList
-# BinaryOp := * | - | ᑌ | ᑎ | ÷ | ᐅᐊ | ᐅLEFTᐊ | ᐅRIGHTᐊ | ᐅFULLᐊ
-#
 # Language definition here:
-# https://github.com/ltworf/relational/wiki/Grammar-and-language
-import re
-
+# http://ltworf.github.io/relational/grammar.html
 from relational import rtypes
 
 RELATION = 0
 UNARY = 1
 BINARY = 2
 
-PRODUCT = u'*'
-DIFFERENCE = u'-'
-UNION = u'∪'
-INTERSECTION = u'∩'
-DIVISION = u'÷'
-JOIN = u'⋈'
-JOIN_LEFT = u'⧑'
-JOIN_RIGHT = u'⧒'
-JOIN_FULL = u'⧓'
-PROJECTION = u'π'
-SELECTION = u'σ'
-RENAME = u'ρ'
-ARROW = u'➡'
+PRODUCT = '*'
+DIFFERENCE = '-'
+UNION = '∪'
+INTERSECTION = '∩'
+DIVISION = '÷'
+JOIN = '⋈'
+JOIN_LEFT = '⧑'
+JOIN_RIGHT = '⧒'
+JOIN_FULL = '⧓'
+PROJECTION = 'π'
+SELECTION = 'σ'
+RENAME = 'ρ'
+ARROW = '➡'
 
 b_operators = (PRODUCT, DIFFERENCE, UNION, INTERSECTION, DIVISION,
                JOIN, JOIN_LEFT, JOIN_RIGHT, JOIN_FULL)  # List of binary operators
@@ -80,17 +62,42 @@ class ParserException (Exception):
     pass
 
 
-class node (object):
+class CallableString(str):
 
-    '''This class is a node of a relational expression. Leaves are relations and internal nodes are operations.
+    '''
+    This is a string. However it is also callable.
 
-    The kind property says if the node is a binary operator, unary operator or relation.
-    Since relations are leaves, a relation node will have no attribute for children.
+    For example:
+    CallableString('1+1')()
+    returns 2
+
+    It is used to contain Python expressions and print
+    or execute them.
+    '''
+
+    def __call__(self, context=None):
+        '''
+        context is a dictionary where to
+        each name is associated the relative relation
+        '''
+        return eval(self, context)
+
+
+class Node (object):
+
+    '''This class is a node of a relational expression. Leaves are relations
+    and internal nodes are operations.
+
+    The 'kind' property indicates whether the node is a binary operator, unary
+    operator or relation.
+    Since relations are leaves, a relation node will have no attribute for
+    children.
 
     If the node is a binary operator, it will have left and right properties.
 
-    If the node is a unary operator, it will have a child, pointing to the child node and a prop containing
-    the string with the props of the operation.
+    If the node is a unary operator, it will have a child, pointing to the
+    child node and a property containing the string with the props of the
+    operation.
 
     This class is used to convert an expression into python code.'''
     kind = None
@@ -116,16 +123,16 @@ class node (object):
                     u"'%s' is not a valid relation name" % self.name)
             return
 
-        '''Expression from right to left, searching for binary operators
-        this means that binary operators have lesser priority than
-        unary operators.
-        It finds the operator with lesser priority, uses it as root of this
-        (sub)tree using everything on its left as left parameter (so building
-        a left subtree with the part of the list located on left) and doing
-        the same on right.
-        Since it searches for strings, and expressions into parenthesis are
-        within sub-lists, they won't be found here, ensuring that they will
-        have highest priority.'''
+        # Expression from right to left, searching for binary operators
+        # this means that binary operators have lesser priority than
+        # unary operators.
+        # It finds the operator with lesser priority, uses it as root of this
+        # (sub)tree using everything on its left as left parameter (so building
+        # a left subtree with the part of the list located on left) and doing
+        # the same on right.
+        # Since it searches for strings, and expressions into parenthesis are
+        # within sub-lists, they won't be found here, ensuring that they will
+        # have highest priority.
         for i in range(len(expression) - 1, -1, -1):
             if expression[i] in b_operators:  # Binary operator
                 self.kind = BINARY
@@ -156,17 +163,25 @@ class node (object):
                 self.child = node(expression[2 + i])
 
                 return
-        raise ParserException(u"Unable to parse tokens")
-        pass
+        raise ParserException("Expected operator in '%s'" % expression)
 
     def toCode(self):
-        '''This method converts the tree into a python code object'''
-        code = self.toPython()
+        '''This method converts the AST into a python code object'''
+        code = self._toPython()
         return compile(code, '<relational_expression>', 'eval')
 
     def toPython(self):
-        '''This method converts the expression into a python code string, which
-        will require the relation module to be executed.'''
+        '''This method converts the AST into a python code string, which
+        will require the relation module to be executed.
+
+        The return value is a CallableString, which means that it can be
+        directly called.'''
+        return CallableString(self._toPython())
+
+    def _toPython(self):
+        '''
+        Same as toPython but returns a regular string
+        '''
         if self.name in b_operators:
             return '%s.%s(%s)' % (self.left.toPython(), op_functions[self.name], self.right.toPython())
         elif self.name in u_operators:
@@ -182,9 +197,7 @@ class node (object):
                 prop = '\"%s\"' % prop
 
             return '%s.%s(%s)' % (self.child.toPython(), op_functions[self.name], prop)
-        else:
-            return self.name
-        pass
+        return self.name
 
     def printtree(self, level=0):
         '''returns a representation of the tree using indentation'''
@@ -202,7 +215,7 @@ class node (object):
         return '\n' + r
 
     def get_left_leaf(self):
-        '''This function returns the leftmost leaf in the tree. It is needed by some optimizations.'''
+        '''This function returns the leftmost leaf in the tree.'''
         if self.kind == RELATION:
             return self
         elif self.kind == UNARY:
@@ -291,36 +304,17 @@ def _find_matching_parenthesis(expression, start=0, openpar=u'(', closepar=u')')
 
 
 def tokenize(expression):
-    '''This function converts an expression into a list where
+    '''This function converts a relational expression into a list where
     every token of the expression is an item of a list. Expressions into
     parenthesis will be converted into sublists.'''
+
     items = []  # List for the tokens
 
-    '''This is a state machine. Initial status is determined by the starting of the
-    expression. There are the following statuses:
-
-    relation: this is the status if the expressions begins with something else than an
-        operator or a parenthesis.
-    binary operator: this is the status when parsing a binary operator, nothing much to say
-    unary operator: this status is more complex, since it will be followed by a parameter AND a
-        sub-expression.
-    sub-expression: this status is entered when finding a '(' and will be exited when finding a ')'.
-        means that the others open must be counted to determine which close is the right one.'''
-
-    expression = expression.strip()  # Removes initial and endind spaces
-    state = 0
-    '''
-    0 initial and useless
-    1 previous stuff was a relation
-    2 previous stuff was a sub-expression
-    3 previous stuff was a unary operator
-    4 previous stuff was a binary operator
-    '''
+    expression = expression.strip()  # Removes initial and ending spaces
 
     while len(expression) > 0:
 
         if expression.startswith('('):  # Parenthesis state
-            state = 2
             end = _find_matching_parenthesis(expression)
             if end == None:
                 raise TokenizerException(
@@ -330,7 +324,7 @@ def tokenize(expression):
             # Removes the entire parentesis and content from the expression
             expression = expression[end + 1:].strip()
 
-        elif expression.startswith((SELECTION, RENAME, PROJECTION)):  # Unary 2 bytes
+        elif expression.startswith((SELECTION, RENAME, PROJECTION)):  # Unary operators
             items.append(expression[0:1])
                          # Adding operator in the top of the list
             expression = expression[
@@ -346,63 +340,30 @@ def tokenize(expression):
                          # Inserting parameter of the operator
             expression = expression[
                 par:].strip()  # Removing parameter from the expression
-        elif expression.startswith((DIVISION, INTERSECTION, UNION, PRODUCT, DIFFERENCE, JOIN, JOIN_LEFT, JOIN_RIGHT, JOIN_FULL)):
-            items.append(expression[0])
-            expression = expression[1:].strip()  # 1 char from the expression
-            state = 4
-        elif re.match(r'[_0-9A-Za-z]', expression[0]) == None:  # At this point we only have relation names, so we raise errors for anything else
-            raise TokenizerException(
-                "Unexpected '%c' in '%s'" % (expression[0], expression))
         else:  # Relation (hopefully)
-            if state == 1:  # Previous was a relation, appending to the last token
-                i = items.pop()
-                items.append(i + expression[0])
-                expression = expression[
-                    1:].strip()  # 1 char from the expression
-            else:
-                state = 1
-                items.append(expression[0])
-                expression = expression[
-                    1:].strip()  # 1 char from the expression
+            expression += ' '  # To avoid the special case of the ending
+
+            # Initial part is a relation, stop when the name of the relation is
+            # over
+            for r in range(1, len(expression)):
+                if rtypes.RELATION_NAME_REGEXP.match(expression[:r + 1]) is None:
+                    break
+            items.append(expression[:r])
+            expression = expression[r:].strip()
 
     return items
 
 
 def tree(expression):
-    '''This function parses a relational algebra expression into a tree and returns
-    the root node using the Node class defined in this module.'''
+    '''This function parses a relational algebra expression into a AST and returns
+    the root node using the Node class.'''
     return node(tokenize(expression))
 
 
 def parse(expr):
-    '''This function parses a relational algebra expression, converting it into python,
-    executable by eval function to get the result of the expression.
-    It has 2 class of operators:
-    without parameters
-    *, -, ᑌ, ᑎ, ᐅᐊ, ᐅLEFTᐊ, ᐅRIGHTᐊ, ᐅFULLᐊ
-    with parameters:
-    σ, π, ρ
-
-    Syntax for operators without parameters is:
-    relation operator relation
-
-    Syntax for operators with parameters is:
-    operator parameters (relation)
-
-    Since a*b is a relation itself, you can parse π a,b (a*b).
-    And since π a,b (A) is a relation, you can parse π a,b (A) ᑌ B.
-
-    You can use parenthesis to change priority: a ᐅᐊ (q ᑌ d).
-
-    IMPORTANT: all strings must be unicode
-
-    EXAMPLES
-    σage > 25 and rank == weight(A)
-    Q ᐅᐊ π a,b(A) ᐅᐊ B
-    ρid➡i,name➡n(A) - π a,b(π a,b(A)) ᑎ σage > 25 or rank = weight(A)
-    π a,b(π a,b(A))
-    ρid➡i,name➡n(π a,b(A))
-    A ᐅᐊ B
+    '''This function parses a relational algebra expression, and returns a
+    CallableString (a string that can be called) whith the corresponding
+    Python expression.
     '''
     return tree(expr).toPython()
 
@@ -411,7 +372,5 @@ if __name__ == "__main__":
         e = input("Expression: ")
         print (parse(e))
 
-    # b=u"σ age>1 and skill=='C' (peopleᐅᐊskills)"
-    # print b[0]
-    # parse(b)
-    pass
+# Backwards compatibility
+node = Node
