@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Relational
 # Copyright (C) 2008  Salvo "LtWorf" Tomaselli
 #
@@ -27,44 +26,65 @@ from collections import deque
 from relational.rtypes import *
 
 
-class relation (object):
+class Relation (object):
 
-    '''This objects defines a relation (as a group of consistent tuples) and operations
-    A relation can be represented using a table
-    Calling an operation and providing a non relation parameter when it is expected will
-    result in a None value'''
+    '''
+    This object defines a relation (as a group of consistent tuples) and operations.
+
+    A relation is a particular kind of set, which has a number of named attributes and
+    a number of tuples, which must express a value for every attribute.
+
+    Set operations like union, intersection and difference are restricted and can only be
+    performed on relations which share the same set of named attributes.
+
+    The constructor optionally accepts a filename and then it will load the relation from
+    that file.
+
+    If no parameter is supplied an empty relation is created.
+
+    Files need to be comma separated as described in RFC4180.
+
+    The first line need to contain the attributes of the relation while the
+    following lines contain the tuples of the relation.
+
+    An empty relation needs a header, and can be filled using the insert()
+    method.
+    '''
     __hash__ = None
 
     def __init__(self, filename=""):
-        '''Creates a relation, accepts a filename and then it will load the relation from
-        that file. If no parameter is supplied an empty relation is created. Empty
-        relations are used in internal operations.
-        By default the file will be handled like a comma separated as described in
-        RFC4180.'''
-
         self._readonly = False
+        self.content = set()
 
         if len(filename) == 0:  # Empty relation
-            self.content = set()
-            self.header = header([])
+            self.header = Header([])
             return
         with open(filename) as fp:
             reader = csv.reader(fp)  # Creating a csv reader
-            self.header = header(next(reader))  # read 1st line
-            self.content = set()
-            attributes = len(self.header)
-
+            self.header = Header(next(reader))  # read 1st line
             iterator = ((self.insert(i) for i in reader))
             deque(iterator, maxlen=0)
 
-    def _make_writable(self):
+    def _make_duplicate(self, copy):
+        '''Flag that the relation "copy" is pointing
+        to the same set as this relation.'''
+
+        self._readonly = True
+        copy._readonly = True
+
+    def _make_writable(self, copy_content=True):
         '''If this relation is marked as readonly, this
-        method will copy the content to make it writable too'''
+        method will copy the content to make it writable too
+
+        if copy_content is set to false, the caller must
+        separately copy the content.'''
 
         if self._readonly:
-            self.content = set(self.content)
             self._readonly = False
 
+            if copy_content:
+                self.content = set(self.content)
+
     def __iter__(self):
         return iter(self.content)
 
@@ -72,50 +92,54 @@ class relation (object):
         return key in self.content
 
     def save(self, filename):
-        '''Saves the relation in a file. By default will save using the csv
-        format as defined in RFC4180, but setting comma_separated to False,
-        it will use the old format with space separated values.
+        '''
+        Saves the relation in a file. Will save using the csv
+        format as defined in RFC4180.
         '''
 
-        fp = open(filename, 'w')  # Opening file in write mode
+        with open(filename, 'w') as fp:
+            writer = csv.writer(fp)  # Creating csv writer
 
-        writer = csv.writer(fp)  # Creating csv writer
+            # It wants an iterable containing iterables
+            head = (self.header,)
+            writer.writerows(head)
 
-        # It wants an iterable containing iterables
-        head = (self.header,)
-        writer.writerows(head)
+            # Writing content, already in the correct format
+            writer.writerows(self.content)
 
-        # Writing content, already in the correct format
-        writer.writerows(self.content)
-        fp.close()  # Closing file
-
-    def _rearrange_(self, other):
+    def _rearrange(self, other):
         '''If two relations share the same attributes in a different order, this method
         will use projection to make them have the same attributes' order.
         It is not exactely related to relational algebra. Just a method used
         internally.
-        Will return None if they don't share the same attributes'''
-        if (self.__class__ != other.__class__):
-            raise Exception('Expected an instance of the same class')
-        if self.header.sharedAttributes(other.header) == len(self.header):
+        Will raise an exception if they don't share the same attributes'''
+        if not isinstance(other, relation):
+            raise TypeError('Expected an instance of the same class')
+        elif self.header == other.header:
+            return other
+        elif self.header.sharedAttributes(other.header) == len(self.header):
             return other.projection(self.header)
-        raise Exception('Relations differ: [%s] [%s]' % (
-            ','.join(self.header) , ','.join(other.header)
-            ))
+        raise TypeError('Relations differ: [%s] [%s]' % (
+            ','.join(self.header), ','.join(other.header)
+        ))
 
     def selection(self, expr):
-        '''Selection, expr must be a valid boolean expression, can contain field names,
-        constant, math operations and boolean ones.'''
-        attributes = {}
+        '''
+        Selection, expr must be a valid Python expression; can contain field names.
+        '''
         newt = relation()
-        newt.header = header(self.header)
+        newt.header = Header(self.header)
+
+        c_expr = compile(expr, 'selection', 'eval')
+
         for i in self.content:
             # Fills the attributes dictionary with the values of the tuple
-            for j,attr in enumerate(self.header):
-                attributes[attr] = i[j].autocast()
+            attributes = {attr: i[j].autocast()
+                          for j, attr in enumerate(self.header)
+                          }
 
             try:
-                if eval(expr, attributes):
+                if eval(c_expr, attributes):
                     newt.content.add(i)
             except Exception as e:
                 raise Exception(
@@ -123,16 +147,18 @@ class relation (object):
         return newt
 
     def product(self, other):
-        '''Cartesian product, attributes must be different to avoid collisions
-        Doing this operation on relations with colliding attributes will
-        cause an exception.
-        It is possible to use rename on attributes and then use the product'''
+        '''
+        Cartesian product. Attributes of the relations must differ.
+        '''
 
-        if (self.__class__ != other.__class__)or(self.header.sharedAttributes(other.header) != 0):
+        if (not isinstance(other, relation)):
+            raise Exception('Operand must be a relation')
+        if self.header.sharedAttributes(other.header) != 0:
             raise Exception(
-                'Unable to perform product on relations with colliding attributes')
+                'Unable to perform product on relations with colliding attributes'
+            )
         newt = relation()
-        newt.header = header(self.header + other.header)
+        newt.header = Header(self.header + other.header)
 
         for i in self.content:
             for j in other.content:
@@ -140,10 +166,17 @@ class relation (object):
         return newt
 
     def projection(self, * attributes):
-        '''Projection operator, takes many parameters, for each field to use.
-        Can also use a single parameter with a list.
-        Will delete duplicate items
-        If an empty list or no parameters are provided, returns None'''
+        '''
+        Can be called in two different ways:
+        a.projection('field1','field2')
+
+        or
+
+        a.projection(['field1','field2'])
+
+        The cardinality of the result, might be less than the cardinality
+        of the original object.
+        '''
         # Parameters are supplied in a list, instead with multiple parameters
         if not isinstance(attributes[0], str):
             attributes = attributes[0]
@@ -155,7 +188,7 @@ class relation (object):
         newt = relation()
         # Create the header
         h = (self.header[i] for i in ids)
-        newt.header = header(h)
+        newt.header = Header(h)
 
         # Create the body
         for i in self.content:
@@ -164,29 +197,30 @@ class relation (object):
         return newt
 
     def rename(self, params):
-        '''Operation rename. Takes a dictionary
-        Will replace the itmem with its content.
-        For example if you want to rename a to b, provide {"a":"b"}
         '''
-        result = []
+        Takes a dictionary.
 
+        Will replace the field name as the key with its value.
+
+        For example if you want to rename a to b, call
+        rel.rename({'a':'b'})
+        '''
         newt = relation()
         newt.header = self.header.rename(params)
 
         newt.content = self.content
-        newt._readonly = True
-        self._readonly = True
+        self._make_duplicate(newt)
         return newt
 
     def intersection(self, other):
-        '''Intersection operation. The result will contain items present in both
+        '''
+        Intersection operation. The result will contain items present in both
         operands.
         Will return an empty one if there are no common items.
-        Will return None if headers are different.
-        It is possible to use projection and rename to make headers match.'''
-        other = self._rearrange_(other)  # Rearranges attributes' order
+        '''
+        other = self._rearrange(other)  # Rearranges attributes' order
         newt = relation()
-        newt.header = header(self.header)
+        newt.header = Header(self.header)
 
         newt.content = self.content.intersection(other.content)
         return newt
@@ -194,12 +228,10 @@ class relation (object):
     def difference(self, other):
         '''Difference operation. The result will contain items present in first
         operand but not in second one.
-        Will return an empty one if the second is a superset of first.
-        Will return None if headers are different.
-        It is possible to use projection and rename to make headers match.'''
-        other = self._rearrange_(other)  # Rearranges attributes' order
+        '''
+        other = self._rearrange(other)  # Rearranges attributes' order
         newt = relation()
-        newt.header = header(self.header)
+        newt.header = Header(self.header)
 
         newt.content = self.content.difference(other.content)
         return newt
@@ -216,18 +248,16 @@ class relation (object):
         # d_headers are the headers from self that aren't also headers in other
         d_headers = tuple(set(self.header) - set(other.header))
 
-        '''
-        Wikipedia defines the division as follows:
+        # Wikipedia defines the division as follows:
 
-        a1,....,an are the d_headers
+        # a1,....,an are the d_headers
 
-        T := πa1,...,an(R) × S
-        U := T - R
-        V := πa1,...,an(U)
-        W := πa1,...,an(R) - V
+        # T := πa1,...,an(R) × S
+        # U := T - R
+        # V := πa1,...,an(U)
+        # W := πa1,...,an(R) - V
 
-        W is the result that we want
-        '''
+        # W is the result that we want
 
         t = self.projection(d_headers).product(other)
         return self.projection(d_headers).difference(t.difference(self).projection(d_headers))
@@ -235,13 +265,10 @@ class relation (object):
     def union(self, other):
         '''Union operation. The result will contain items present in first
         and second operands.
-        Will return an empty one if both are empty.
-        Will not insert tuplicated items.
-        Will return None if headers are different.
-        It is possible to use projection and rename to make headers match.'''
-        other = self._rearrange_(other)  # Rearranges attributes' order
+        '''
+        other = self._rearrange(other)  # Rearranges attributes' order
         newt = relation()
-        newt.header = header(self.header)
+        newt.header = Header(self.header)
 
         newt.content = self.content.union(other.content)
         return newt
@@ -258,19 +285,19 @@ class relation (object):
         return a.union(b)
 
     def outer_right(self, other):
-        '''Outer right join. Considers self as left and param as right. If the
+        '''
+        Outer right join. Considers self as left and param as right. If the
         tuple has no corrispondence, empy attributes are filled with a "---"
-        string. This is due to the fact that empty string or a space would cause
-        problems when saving the relation.
-        Just like natural join, it works considering shared attributes.'''
+        string. This is due to the fact that the None token would cause
+        problems when saving and reloading the relation.
+        Just like natural join, it works considering shared attributes.
+        '''
         return other.outer_left(self)
 
     def outer_left(self, other, swap=False):
-        '''Outer left join. Considers self as left and param as right. If the
-        tuple has no corrispondence, empty attributes are filled with a "---"
-        string. This is due to the fact that empty string or a space would cause
-        problems when saving the relation.
-        Just like natural join, it works considering shared attributes.'''
+        '''
+        See documentation for outer_right
+        '''
 
         shared = self.header.intersection(other.header)
 
@@ -278,7 +305,7 @@ class relation (object):
         # Creating the header with all the fields, done like that because order is
         # needed
         h = (i for i in other.header if i not in shared)
-        newt.header = header(chain(self.header,h))
+        newt.header = Header(chain(self.header, h))
 
         # Shared ids of self
         sid = self.header.getAttributesId(shared)
@@ -297,20 +324,22 @@ class relation (object):
                     match = match and (i[sid[k]] == j[oid[k]])
 
                 if match:
-                    item = chain(i,(j[l] for l in noid))
+                    item = chain(i, (j[l] for l in noid))
 
                     newt.content.add(tuple(item))
                     added = True
             # If it didn't partecipate, adds it
             if not added:
-                item = chain(i,repeat('---',len(noid)))
+                item = chain(i, repeat(rstring('---'), len(noid)))
                 newt.content.add(tuple(item))
 
         return newt
 
     def join(self, other):
-        '''Natural join, joins on shared attributes (one or more). If there are no
-        shared attributes, it will behave as cartesian product.'''
+        '''
+        Natural join, joins on shared attributes (one or more). If there are no
+        shared attributes, it will behave as the cartesian product.
+        '''
 
         # List of attributes in common between the relations
         shared = self.header.intersection(other.header)
@@ -320,7 +349,7 @@ class relation (object):
         # Creating the header with all the fields, done like that because order is
         # needed
         h = (i for i in other.header if i not in shared)
-        newt.header = header(chain(self.header,h))
+        newt.header = Header(chain(self.header, h))
 
         # Shared ids of self
         sid = self.header.getAttributesId(shared)
@@ -343,16 +372,17 @@ class relation (object):
         return newt
 
     def __eq__(self, other):
-        '''Returns true if the relations are the same, ignoring order of items.
-        This operation is rather heavy, since it requires sorting and comparing.'''
-        if self.__class__ != other.__class__:
+        if not isinstance(other, relation):
+            return False
+
+        if len(self.content) != len(other.content):
             return False
 
         if set(self.header) != set(other.header):
             return False
 
         # Rearranges attributes' order so can compare tuples directly
-        other = self._rearrange_(other)
+        other = self._rearrange(other)
 
         # comparing content
         return self.content == other.content
@@ -361,69 +391,65 @@ class relation (object):
         return len(self.content)
 
     def __str__(self):
-        '''Returns a string representation of the relation, can be printed with
-        monospaced fonts'''
-        m_len = []  # Maximum lenght string
-        for f in self.header:
-            m_len.append(len(f))
+        m_len = [len(i) for i in self.header]  # Maximum lenght string
 
         for f in self.content:
-            col = 0
-            for i in f:
+            for col, i in enumerate(f):
                 if len(i) > m_len[col]:
                     m_len[col] = len(i)
-                col += 1
 
         res = ""
-        for f,attr in enumerate(self.header):
+        for f, attr in enumerate(self.header):
             res += "%s" % (attr.ljust(2 + m_len[f]))
 
         for r in self.content:
-            col = 0
             res += "\n"
-            for i in r:
+            for col, i in enumerate(r):
                 res += "%s" % (i.ljust(2 + m_len[col]))
-                col += 1
 
         return res
 
     def update(self, expr, dic):
-        '''Update, expr must be a valid boolean expression, can contain field names,
-        constant, math operations and boolean ones.
+        '''
+        Updates certain values of a relation.
+
+        expr must be a valid Python expression that can contain field names.
+
         This operation will change the relation itself instead of generating a new one,
-        updating all the tuples that make expr true.
-        Dic must be a dictionary that has the form field name:value. Every kind of value
+        updating all the tuples where expr evaluates as True.
+
+        Dic must be a dictionary that has the form "field name":"new value". Every kind of value
         will be converted into a string.
-        Returns the number of affected rows.'''
-        self._make_writable()
-        affected = 0
-        attributes = {}
-        keys = dic.keys()  # List of headers to modify
-        f_ids = self.header.getAttributesId(
-            keys)  # List of indexes corresponding to keys
 
-        # new_content=[] #New content of the relation
-        for i in self.content:
-            for j,attr in enumerate(self.header):
-                attributes[attr] = i[j].autocast()
+        Returns the number of affected rows.
+        '''
+        self._make_writable(copy_content=False)
+        affected = self.selection(expr)
+        not_affected = self.difference(affected)
 
-            if eval(expr, attributes):  # If expr is true, changing the tuple
-                affected += 1
-                new_tuple = list(i)
-                # Deleting the tuple, instead of changing it, so other
-                # relations can still point to the same list without
-                # being affected.
-                self.content.remove(i)
-                for k in range(len(keys)):
-                    new_tuple[f_ids[k]] = rstring(dic[keys[k]])
-                self.content.add(tuple(new_tuple))
-        return affected
+        new_values = tuple(
+            zip(self.header.getAttributesId(dic.keys()), dic.values())
+        )
+
+        for i in set(affected.content):
+            i = list(i)
+
+            for column, value in new_values:
+                i[column] = value
+            not_affected.insert(i)
+
+        self.content = not_affected.content
+        return len(affected)
 
     def insert(self, values):
-        '''Inserts a tuple in the relation.
+        '''
+        Inserts a tuple in the relation.
         This function will not insert duplicate tuples.
         All the values will be converted in string.
-        Will return the number of inserted rows.'''
+        Will return the number of inserted rows.
+
+        Will fail if the tuple has the wrong amount of items.
+        '''
 
         if len(self.header) != len(values):
             raise Exception(
@@ -440,28 +466,27 @@ class relation (object):
         return len(self.content) - prevlen
 
     def delete(self, expr):
-        '''Delete, expr must be a valid boolean expression, can contain field names,
-        constant, math operations and boolean ones.
+        '''
+        Delete, expr must be a valid Python expression; can contain field names.
+
         This operation will change the relation itself instead of generating a new one,
-        deleting all the tuples that make expr true.
+        deleting all the tuples where expr evaluates as True.
+
         Returns the number of affected rows.'''
 
-        #Not necessary self._make_writable()
-
         l = len(self.content)
-        self._readonly = False
+        self._make_writable(copy_content=False)
         self.content = self.difference(self.selection(expr)).content
         return len(self.content) - l
 
 
-class header(tuple):
+class Header(tuple):
 
     '''This class defines the header of a relation.
     It is used within relations to know if requested operations are accepted'''
 
-    # Since relations are mutalbe we explicitly block hashing them
-    def __new__ (cls, fields):
-        return super(header, cls).__new__(cls, tuple(fields))
+    def __new__(cls, fields):
+        return super(Header, cls).__new__(cls, tuple(fields))
 
     def __init__(self, *args, **kwargs):
         '''Accepts a list with attributes' names. Names MUST be unique'''
@@ -474,7 +499,7 @@ class header(tuple):
             raise Exception('Attribute names must be unique')
 
     def __repr__(self):
-        return "header(%s)" % super(header, self).__repr__()
+        return "Header(%s)" % super(Header, self).__repr__()
 
     def rename(self, params):
         '''Returns a new header, with renamed fields.
@@ -482,7 +507,7 @@ class header(tuple):
         params is a dictionary of {old:new} names
         '''
         attrs = list(self)
-        for old,new in params.items():
+        for old, new in params.items():
             if not is_valid_relation_name(new):
                 raise Exception('%s is not a valid attribute name' % new)
             try:
@@ -490,7 +515,7 @@ class header(tuple):
                 attrs[id_] = new
             except:
                 raise Exception('Field not found: %s' % old)
-        return header(attrs)
+        return Header(attrs)
 
     def sharedAttributes(self, other):
         '''Returns how many attributes this header has in common with a given one'''
@@ -507,3 +532,7 @@ class header(tuple):
     def getAttributesId(self, param):
         '''Returns a list with numeric index corresponding to field's name'''
         return [self.index(i) for i in param]
+
+# Backwards compatibility
+relation = Relation
+header = Header
