@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Relational
 # Copyright (C) 2008  Salvo "LtWorf" Tomaselli
 #
@@ -24,10 +23,19 @@
 import datetime
 import re
 
+RELATION_NAME_REGEXP = re.compile(r'^[_a-z][_a-z0-9]*$', re.IGNORECASE)
 
-class rstring (str):
+
+class Rstring (str):
 
     '''String subclass with some custom methods'''
+
+    int_regexp = re.compile(r'^[\+\-]{0,1}[0-9]+$')
+    float_regexp = re.compile(r'^[\+\-]{0,1}[0-9]+(\.([0-9])+)?$')
+    date_regexp = re.compile(
+        r'^([0-9]{1,4})(\\|-|/)([0-9]{1,2})(\\|-|/)([0-9]{1,2})$'
+    )
+
     def autocast(self):
         '''
         Returns the automatic cast for this
@@ -54,10 +62,7 @@ class rstring (str):
         the following regexp:
         r'^[\+\-]{0,1}[0-9]+$'
         '''
-        if re.match(r'^[\+\-]{0,1}[0-9]+$', self) == None:
-            return False
-        else:
-            return True
+        return Rstring.int_regexp.match(self) is not None
 
     def isFloat(self):
         '''Returns true if the string represents a float number
@@ -65,10 +70,7 @@ class rstring (str):
         the following regexp:
             r'^[\+\-]{0,1}[0-9]+(\.([0-9])+)?$'
         '''
-        if re.match(r'^[\+\-]{0,1}[0-9]+(\.([0-9])+)?$', self) == None:
-            return False
-        else:
-            return True
+        return Rstring.float_regexp.match(self) is not None
 
     def isDate(self):
         '''Returns true if the string represents a date,
@@ -81,9 +83,8 @@ class rstring (str):
         except:
             pass
 
-        r = re.match(
-            r'^([0-9]{1,4})(\\|-|/)([0-9]{1,2})(\\|-|/)([0-9]{1,2})$', self)
-        if r == None:
+        r = Rstring.date_regexp.match(self)
+        if r is None:
             self._isdate = False
             self._date = None
             return False
@@ -110,7 +111,7 @@ class rstring (str):
             return self._date
 
 
-class rdate (object):
+class Rdate (object):
 
     '''Represents a date'''
 
@@ -160,7 +161,8 @@ class rdate (object):
 def is_valid_relation_name(name):
     '''Checks if a name is valid for a relation.
     Returns boolean'''
-    if re.match(r'^[_a-zA-Z]+[_a-zA-Z0-9]*$', name) == None:
-        return False
-    else:
-        return True
+    return re.match(RELATION_NAME_REGEXP, name) != None
+
+# Backwards compatibility
+rdate = Rdate
+rstring = Rstring
