@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Relational
 # Copyright (C) 2008  Salvo "LtWorf" Tomaselli
 #
@@ -17,7 +16,7 @@
 #
 # author Salvo "LtWorf" Tomaselli <tiposchi@tiscali.it>
 
-from PyQt5 import QtCore, QtGui, QtWidgets
+from PyQt5 import QtGui, QtWidgets
 
 from relational_gui import rel_edit
 from relational import relation
@@ -81,7 +80,8 @@ class creatorForm(QtWidgets.QDialog):
         self.table.setItem(1, 1, i11)
 
     def create_relation(self):
-        h = (self.table.item(0, i).text() for i in range(self.table.columnCount()))
+        h = (self.table.item(0, i).text()
+             for i in range(self.table.columnCount()))
 
         try:
             header = relation.header(h)
