@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Relational
 # Copyright (C) 2008  Salvo "LtWorf" Tomaselli
 #
@@ -17,12 +16,11 @@
 #
 # author Salvo "LtWorf" Tomaselli <tiposchi@tiscali.it>
 import sys
-import os
-import os.path
 
-from PyQt5 import QtCore, QtWidgets, QtWidgets
+from PyQt5 import QtCore, QtWidgets, QtGui
 
-from relational import relation, parser, optimizer, rtypes
+from relational import parser, optimizer, rtypes
+from relational.maintenance import UserInterface
 
 from relational_gui import about
 from relational_gui import survey
@@ -32,17 +30,81 @@ from relational_gui import maingui
 
 class relForm(QtWidgets.QMainWindow):
 
-    def __init__(self, ui):
+    def __init__(self):
         QtWidgets.QMainWindow.__init__(self)
         self.About = None
         self.Survey = None
-        self.relations = {}  # Dictionary for relations
         self.undo = None  # UndoQueue for queries
         self.selectedRelation = None
-        self.ui = ui
-        self.qcounter = 1  # Query counter
+        self.ui = maingui.Ui_MainWindow()
+        self.user_interface = UserInterface()
+        self.history_current_line = None
+
+        # Creates the UI
+        self.ui.setupUi(self)
+
+        # Setting fonts for symbols
+        f = QtGui.QFont()
+        size = f.pointSize()
+        if sys.platform.startswith('win'):
+            winFont = 'Cambria'
+            symbolFont = 'Segoe UI Symbol'
+            increment = 4
+        else:
+            winFont = f.family()
+            symbolFont = f.family()
+            increment = 2
+        font = QtGui.QFont(winFont, size + increment)
+        sfont = QtGui.QFont(symbolFont)
+        self.ui.lstHistory.setFont(font)
+        self.ui.txtMultiQuery.setFont(font)
+        self.ui.txtQuery.setFont(font)
+        self.ui.groupOperators.setFont(font)
+        self.ui.cmdClearMultilineQuery.setFont(sfont)
+        self.ui.cmdClearQuery.setFont(sfont)
 
         self.settings = QtCore.QSettings()
+        self._restore_settings()
+
+
+        # Shortcuts
+        shortcuts = (
+            (self.ui.lstRelations, QtGui.QKeySequence.Delete, self.unloadRelation),
+            (self.ui.lstRelations, 'Space', lambda: self.printRelation(self.ui.lstRelations.currentItem())),
+            (self.ui.txtQuery, QtGui.QKeySequence.MoveToNextLine, self.next_history),
+            (self.ui.txtQuery, QtGui.QKeySequence.MoveToPreviousLine, self.prev_history),
+        )
+
+        self.add_shortcuts(shortcuts)
+
+    def next_history(self):
+        if self.ui.lstHistory.currentRow() + 1 == self.ui.lstHistory.count() and self.history_current_line:
+            self.ui.txtQuery.setText(self.history_current_line)
+            self.history_current_line = None
+        elif self.history_current_line:
+            self.ui.lstHistory.setCurrentRow(self.ui.lstHistory.currentRow()+1)
+            self.resumeHistory(self.ui.lstHistory.currentItem())
+
+    def prev_history(self):
+        if self.history_current_line is None:
+            self.history_current_line = self.ui.txtQuery.text()
+
+            if self.ui.lstHistory.currentItem() is None:
+                return
+            if not self.ui.lstHistory.currentItem().text() != self.ui.txtQuery.text():
+                self.ui.lstHistory.setCurrentRow(self.ui.lstHistory.currentRow()-1)
+        elif self.ui.lstHistory.currentRow() > 0:
+            self.ui.lstHistory.setCurrentRow(self.ui.lstHistory.currentRow()-1)
+        self.resumeHistory(self.ui.lstHistory.currentItem())
+
+    def add_shortcuts(self, shortcuts):
+        for widget,shortcut,slot in shortcuts:
+            action = QtWidgets.QAction(self)
+            action.triggered.connect(slot)
+            action.setShortcut(QtGui.QKeySequence(shortcut))
+            # I couldn't find the constant
+            action.setShortcutContext(0)
+            widget.addAction(action)
 
     def checkVersion(self):
         from relational import maintenance
@@ -61,12 +123,20 @@ class relForm(QtWidgets.QMainWindow):
         QtWidgets.QMessageBox.information(
             self, QtWidgets.QApplication.translate("Form", "Version"), r)
 
+    def setHistoryShown(self, history_shown):
+        self.history_shown = history_shown
+        self.settings.setValue('history_shown', history_shown)
+        self.ui.lstHistory.setVisible(history_shown)
+        self.ui.actionShow_history.setChecked(history_shown)
+
     def setMultiline(self, multiline):
         self.multiline = multiline
         self.settings.setValue('multiline', multiline)
-        self.ui.lineExpressionFrame.setVisible(not multiline)
-        self.ui.frmOptimizations.setVisible(not multiline)
-        self.ui.frmMultiLine.setVisible(multiline)
+        if multiline:
+            index = 0
+        else:
+            index = 1
+        self.ui.stackedWidget.setCurrentIndex(index)
         self.ui.actionMulti_line_mode.setChecked(multiline)
 
     def load_query(self, *index):
@@ -81,95 +151,67 @@ class relForm(QtWidgets.QMainWindow):
         '''Performs all the possible optimizations on the query'''
         self.undo = self.ui.txtQuery.text()  # Storing the query in undo list
 
-        query = self.ui.txtQuery.text()
+        res_rel,query = self.user_interface.split_query(self.ui.txtQuery.text(),None)
         try:
-            result = optimizer.optimize_all(query, self.relations)
+            trace = []
+            result = optimizer.optimize_all(
+                query,
+                self.user_interface.relations,
+                debug=trace
+            )
+            print('==== Optimization steps ====')
+            print(query)
+            print('\n'.join(trace))
+            print('========')
+
+            if res_rel:
+                result = '%s = %s' % (res_rel, result)
             self.ui.txtQuery.setText(result)
         except Exception as e:
-            QtWidgets.QMessageBox.information(None, QtWidgets.QApplication.translate("Form", "Error"), "%s\n%s" %
-                                              (QtWidgets.QApplication.translate("Form", "Check your query!"), e.__str__()))
+            self.error(e)
 
     def resumeHistory(self, item):
-        itm = item.text().split(' = ', 1)
-        self.ui.txtResult.setText(itm[0])
-        self.ui.txtQuery.setText(itm[1])
-
-    def _run_multiline(self):
-        query = self.ui.txtMultiQuery.toPlainText()
-        self.settings.setValue('multiline/query', query)
-
-        queries = query.split('\n')
-
-        for query in queries:
-            if query.strip() == '':
-                continue
-
-            parts = query.split('=', 1)
-            parts[0] = parts[0].strip()
-            if len(parts) > 1 and rtypes.is_valid_relation_name(parts[0].strip()):
-                relname = parts[0].strip()
-                query = parts[1]
-            else:
-                relname = 'last_'
-
-            try:
-                expr = parser.parse(query)
-                print ('%s <- %s' % (relname, expr))
-                result = eval(expr, self.relations)
-                self.relations[relname] = result
-            except Exception as e:
-                print(str(e))
-                QtWidgets.QMessageBox.information(None, QtWidgets.QApplication.translate("Form", "Error"), u"%s\n%s" %
-                                              (QtWidgets.QApplication.translate("Form", "Check your query!"), str(e)))
-                break
-        self.updateRelations()  # update the list
-        self.selectedRelation = result
-        self.showRelation(self.selectedRelation)
+        if item is None:
+            return
+        itm = item.text()
+        self.ui.txtQuery.setText(itm)
 
     def execute(self):
-        '''Executes the query'''
-        if self.multiline:
-            return self._run_multiline()
 
-        #Single line query
-        query = self.ui.txtQuery.text()
-        res_rel = self.ui.txtResult.text()  # result relation's name
-
-        if not rtypes.is_valid_relation_name(res_rel):
-            QtWidgets.QMessageBox.information(self, QtWidgets.QApplication.translate(
-                "Form", "Error"), QtWidgets.QApplication.translate("Form", "Wrong name for destination relation."))
-            return
+        # Show the 'Processing' frame
+        self.ui.stackedWidget.setCurrentIndex(2)
+        QtCore.QCoreApplication.processEvents()
 
         try:
-            # Converting string to utf8 and then from qstring to normal string
-            expr = parser.parse(query)  # Converting expression to python code
-            print (query, "-->", expr)  # Printing debug
-            result = eval(expr, self.relations)  # Evaluating the expression
+            '''Executes the query'''
+            if self.multiline:
+                query = self.ui.txtMultiQuery.toPlainText()
+                self.settings.setValue('multiline/query', query)
+            else:
+                query = self.ui.txtQuery.text()
 
-            self.relations[
-                res_rel] = result  # Add the relation to the dictionary
-            self.updateRelations()  # update the list
-            self.selectedRelation = result
-            self.showRelation(self.selectedRelation)
-                              # Show the result in the table
-        except Exception as e:
-            print (str(e))
-            QtWidgets.QMessageBox.information(None, QtWidgets.QApplication.translate("Form", "Error"), u"%s\n%s" %
-                                              (QtWidgets.QApplication.translate("Form", "Check your query!"), str(e)))
-            return
+            try:
+                self.selectedRelation = self.user_interface.multi_execute(query)
+            except Exception as e:
+                return self.error(e)
+            finally:
+                self.updateRelations()  # update the list
+                self.showRelation(self.selectedRelation)
 
-        # Adds to history
-        item = u'%s = %s' % (
-            self.ui.txtResult.text(),
-            self.ui.txtQuery.text()
-        )
-        hitem = QtWidgets.QListWidgetItem(None, 0)
-        hitem.setText(item)
-        self.ui.lstHistory.addItem(hitem)
-        self.ui.lstHistory.setCurrentItem(hitem)
+            if not self.multiline:
+                # Last in history
+                item = self.ui.lstHistory.item(self.ui.lstHistory.count() - 1)
+
+                if item is None or item.text() != query:
+                    # Adds to history if it is not already the last
+                    hitem = QtWidgets.QListWidgetItem(None, 0)
+                    hitem.setText(query)
+                    self.ui.lstHistory.addItem(hitem)
+                    self.ui.lstHistory.setCurrentItem(hitem)
+        finally:
+            # Restore the normal frame
+            self.setMultiline(self.multiline)
 
-        self.qcounter += 1
-        self.ui.txtResult.setText(u"_last%d" % self.qcounter)  # Sets the result relation name to none
 
     def showRelation(self, rel):
         '''Shows the selected relation into the table'''
@@ -184,30 +226,29 @@ class relForm(QtWidgets.QMainWindow):
         # Set content
         for i in rel.content:
             item = QtWidgets.QTreeWidgetItem()
-            for j in range(len(i)):
-                item.setText(j, i[j])
+            for j,k in enumerate(i):
+                item.setText(j, k)
             self.ui.table.addTopLevelItem(item)
 
         # Sets columns
-        for i,attr in enumerate(rel.header):
+        for i, attr in enumerate(rel.header):
             self.ui.table.headerItem().setText(i, attr)
-            self.ui.table.resizeColumnToContents(
-                i)  # Must be done in order to avoid  too small columns
+            self.ui.table.resizeColumnToContents(i)
 
     def printRelation(self, item):
-        self.selectedRelation = self.relations[item.text()]
+        self.selectedRelation = self.user_interface.relations[item.text()]
         self.showRelation(self.selectedRelation)
 
     def showAttributes(self, item):
         '''Shows the attributes of the selected relation'''
         rel = item.text()
         self.ui.lstAttributes.clear()
-        for j in self.relations[rel].header:
+        for j in self.user_interface.relations[rel].header:
             self.ui.lstAttributes.addItem(j)
 
     def updateRelations(self):
         self.ui.lstRelations.clear()
-        for i in self.relations:
+        for i in self.user_interface.relations:
             if i != "__builtins__":
                 self.ui.lstRelations.addItem(i)
 
@@ -220,26 +261,41 @@ class relForm(QtWidgets.QMainWindow):
                 self, QtWidgets.QApplication.translate("Form", "Error"), r
             )
             return
-        filename = QtWidgets.QFileDialog.getSaveFileName(self, QtWidgets.QApplication.translate(
-            "Form", "Save Relation"), "", QtWidgets.QApplication.translate("Form", "Relations (*.csv)"))[0]
+        filename = QtWidgets.QFileDialog.getSaveFileName(
+            self, QtWidgets.QApplication.translate("Form", "Save Relation"),
+            "",
+            QtWidgets.QApplication.translate("Form", "Relations (*.csv)")
+        )[0]
         if (len(filename) == 0):  # Returns if no file was selected
             return
         self.selectedRelation.save(filename)
 
     def unloadRelation(self):
         for i in self.ui.lstRelations.selectedItems():
-            del self.relations[i.text()]
+            del self.user_interface.relations[i.text()]
+        self.updateRelations()
+
+    def newSession(self):
+        self.user_interface.session_reset()
         self.updateRelations()
 
     def editRelation(self):
         from relational_gui import creator
         for i in self.ui.lstRelations.selectedItems():
             result = creator.edit_relation(
-                self.relations[i.text()])
+                self.user_interface.get_relation(i.text())
+            )
             if result != None:
-                self.relations[i.text()] = result
+                self.user_interface.set_relation(i.text(), result)
         self.updateRelations()
 
+    def error(self, exception):
+        print (exception)
+        QtWidgets.QMessageBox.information(
+            None, QtWidgets.QApplication.translate("Form", "Error"),
+            str(exception)
+        )
+
     def promptRelationName(self):
         while True:
             res = QtWidgets.QInputDialog.getText(
@@ -249,12 +305,13 @@ class relForm(QtWidgets.QMainWindow):
                     "Form", "Insert the name for the new relation"),
                 QtWidgets.QLineEdit.Normal, ''
             )
-            if res[1] == False:# or len(res[0]) == 0:
+            if res[1] == False:  # or len(res[0]) == 0:
                 return None
             name = res[0]
             if not rtypes.is_valid_relation_name(name):
                 r = QtWidgets.QApplication.translate(
-                    "Form", str("Wrong name for destination relation: %s." % name)
+                    "Form", str(
+                        "Wrong name for destination relation: %s." % name)
                 )
                 QtWidgets.QMessageBox.information(
                     self, QtWidgets.QApplication.translate("Form", "Error"), r
@@ -271,14 +328,10 @@ class relForm(QtWidgets.QMainWindow):
         name = self.promptRelationName()
 
         try:
-            self.relations[name] = result
+            self.user_interface.relations[name] = result
             self.updateRelations()
         except Exception as e:
-            print (e)
-            QtWidgets.QMessageBox.information(None, QtWidgets.QApplication.translate("Form", "Error"), "%s\n%s" %
-                                            (QtWidgets.QApplication.translate("Form", "Check your query!"), e.__str__()))
-        finally:
-            return
+            self.error(e)
 
     def closeEvent(self, event):
         self.save_settings()
@@ -287,14 +340,21 @@ class relForm(QtWidgets.QMainWindow):
     def save_settings(self):
         self.settings.setValue('maingui/geometry', self.saveGeometry())
         self.settings.setValue('maingui/windowState', self.saveState())
+        self.settings.setValue('maingui/splitter', self.ui.splitter.saveState())
+        self.settings.setValue('maingui/relations', self.user_interface.session_dump())
 
-    def restore_settings(self):
-        # self.settings.value('session_name','default').toString()
-        self.setMultiline(self.settings.value('multiline','false')=='true')
-        self.ui.txtMultiQuery.setPlainText(self.settings.value('multiline/query',''))
+    def _restore_settings(self):
+        self.user_interface.session_restore(self.settings.value('maingui/relations'))
+        self.updateRelations()
+
+        self.setMultiline(self.settings.value('multiline', 'false') == 'true')
+        self.setHistoryShown(self.settings.value('history_shown', 'true') == 'true')
+        self.ui.txtMultiQuery.setPlainText(
+            self.settings.value('multiline/query', ''))
         try:
             self.restoreGeometry(self.settings.value('maingui/geometry'))
             self.restoreState(self.settings.value('maingui/windowState'))
+            self.ui.splitter.restoreState(self.settings.value('maingui/splitter'))
         except:
             pass
 
@@ -320,32 +380,29 @@ class relForm(QtWidgets.QMainWindow):
         It shouldn't be called giving filename but not giving name.'''
         # Asking for file to load
         if not filenames:
-            f = QtWidgets.QFileDialog.getOpenFileNames(self, QtWidgets.QApplication.translate(
-                "Form", "Load Relation"), "", QtWidgets.QApplication.translate("Form", "Relations (*.csv);;Text Files (*.txt);;All Files (*)"))
+            f = QtWidgets.QFileDialog.getOpenFileNames(
+                self,
+                QtWidgets.QApplication.translate("Form", "Load Relation"),
+                "",
+                QtWidgets.QApplication.translate(
+                    "Form",
+                    "Relations (*.csv);;Text Files (*.txt);;All Files (*)"
+                )
+            )
             filenames = f[0]
 
         for f in filenames:
             # Default relation's name
-            name = os.path.basename(f).lower()
-
-            if len(name) == 0:
-                return
-
-            if (name.endswith(".csv")):  # removes the extension
-                name = name[:-4]
-
-            if not rtypes.is_valid_relation_name(name):
+            name = self.user_interface.suggest_name(f)
+            if name is None:
                 name = self.promptRelationName()
-
             if name is None:
                 continue
 
             try:
-                self.relations[name] = relation.relation(f)
+                self.user_interface.load(f, name)
             except Exception as e:
-                print (e)
-                QtWidgets.QMessageBox.information(None, QtWidgets.QApplication.translate("Form", "Error"), "%s\n%s" %
-                                              (QtWidgets.QApplication.translate("Form", "Check your query!"), e.__str__()))
+                self.error(e)
                 continue
 
         self.updateRelations()
