@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-# coding=UTF-8
 # Relational
 # Copyright (C) 2010  Salvo "LtWorf" Tomaselli
 #
@@ -27,11 +25,21 @@ import os
 import sys
 
 from relational import relation, parser, rtypes
+from relational import maintenance
 from xtermcolor import colorize
 
 PROMPT_COLOR = 0xffff00
 ERROR_COLOR = 0xff0000
 
+TTY = os.isatty(0) and os.isatty(1)
+
+def printtty(*args, **kwargs):
+    '''
+    Prints only if stdout and stdin are a tty
+    '''
+    if TTY:
+        print(*args,**kwargs)
+
 
 class SimpleCompleter(object):
 
@@ -123,7 +131,7 @@ def load_relation(filename, defname=None):
         relations[defname] = relation.relation(filename)
 
         completer.add_completion(defname)
-        print (colorize("Loaded relation %s" % defname, 0x00ff00))
+        printtty(colorize("Loaded relation %s" % defname, 0x00ff00))
         return defname
     except Exception as e:
         print (colorize(e, ERROR_COLOR), file=sys.stderr)
@@ -132,7 +140,6 @@ def load_relation(filename, defname=None):
 
 def survey():
     '''performs a survey'''
-    from relational import maintenance
 
     post = {'software': 'Relational algebra (cli)', 'version': version}
 
@@ -276,20 +283,14 @@ def exec_query(command):
 
     # Finds the name in where to save the query
     parts = command.split('=', 1)
-
-    if len(parts) > 1 and rtypes.is_valid_relation_name(parts[0]):
-        relname = parts[0]
-        query = parts[1]
-    else:
-        relname = 'last_'
-        query = command
+    relname,query = maintenance.UserInterface.split_query(command)
 
     # Execute query
     try:
         pyquery = parser.parse(query)
-        result = eval(pyquery, relations)
+        result = pyquery(relations)
 
-        print (colorize("-> query: %s" % pyquery, 0x00ff00))
+        printtty(colorize("-> query: %s" % pyquery, 0x00ff00))
 
         if printrel:
             print ()
@@ -303,8 +304,8 @@ def exec_query(command):
 
 
 def main(files=[]):
-    print (colorize('> ', PROMPT_COLOR) + "; Type HELP to get the HELP")
-    print (colorize('> ', PROMPT_COLOR) +
+    printtty(colorize('> ', PROMPT_COLOR) + "; Type HELP to get the HELP")
+    printtty(colorize('> ', PROMPT_COLOR) +
            "; Completion is activated using the tab (if supported by the terminal)")
 
     for i in files:
@@ -318,14 +319,18 @@ def main(files=[]):
 
     while True:
         try:
-            line = input(colorize('> ', PROMPT_COLOR))
+
+            line = input(colorize('> ' if TTY else '', PROMPT_COLOR))
             if isinstance(line, str) and len(line) > 0:
                 exec_line(line)
         except KeyboardInterrupt:
-            print ('^C\n')
-            continue
+            if TTY:
+                print ('^C\n')
+                continue
+            else:
+                break
         except EOFError:
-            print ()
+            printtty()
             sys.exit(0)
 
 
